"""Schema definitions."""
# Standard Modules
from enum import Enum

# Local Modules
from cofactr.helpers import identity
from cofactr.schema.flagship.offer import Offer as FlagshipOffer
from cofactr.schema.logistics.offer import Offer as LogisticsOffer
from cofactr.schema.flagship.part import Part as FlagshipPart
from cofactr.schema.logistics.part import Part as LogisticsPart
from cofactr.schema.flagship.seller import Seller as FlagshipSeller


class ProductSchemaName(str, Enum):
    """Product schema name."""

    INTERNAL = "internal"
    FLAGSHIP = "flagship"
    LOGISTICS = "logistics"


schema_to_product = {
    ProductSchemaName.INTERNAL: identity,
    ProductSchemaName.FLAGSHIP: FlagshipPart,
    ProductSchemaName.LOGISTICS: LogisticsPart,
}


class OfferSchemaName(str, Enum):
    """Offer schema name."""

    INTERNAL = "internal"
    FLAGSHIP = "flagship"
    LOGISTICS = "logistics"


schema_to_offer = {
    OfferSchemaName.INTERNAL: identity,
    OfferSchemaName.FLAGSHIP: FlagshipOffer,
    OfferSchemaName.LOGISTICS: LogisticsOffer,
}


class OrgSchemaName(str, Enum):
    """Organization schema name."""

    INTERNAL = "internal"
    FLAGSHIP = "flagship"
    LOGISTICS = "logistics"


schema_to_org = {
    OrgSchemaName.INTERNAL: identity,
    OrgSchemaName.FLAGSHIP: FlagshipSeller,
    OrgSchemaName.LOGISTICS: FlagshipSeller,
}



"""Types defining KB entities."""
# Standard Modules
from datetime import datetime
from typing import Any, Dict, List, Literal, Optional, TypedDict
from typing_extensions import NotRequired

Property = str


class Alias(TypedDict):
    """Alias in KB structure."""

    language: str  # Country code.
    value: str  # Alias name.


Aliases = Dict[str, List[Alias]]

Labels = Dict[str, Alias]

SnakType = Literal["value"]
DataType = Literal[
    "boolean",
    "external_id",
    "monolingual_text",
    "quantity",
    "url",
    "time",
]
# "number", "string", { "amount": number, "unit": str }, etc.
DataValue = Dict[str, Any]


class Snak(TypedDict):
    """S.N.A.K. (Some Notation About Knowledge)."""

    snaktype: SnakType
    property: Property
    datavalue: DataValue
    datatype: DataType


Mainsnak = Snak


Qualifier = Dict[str, List[Snak]]

Rank = Literal["preferred", "normal", "deprecated"]
Type = Literal["statement"]


Snaks = Dict[Property, List[Snak]]
SnaksOrder = List[Property]


class Reference(TypedDict):
    """Reference."""

    snaks: Snaks
    snaks_order: SnaksOrder


class StatementObject(TypedDict):
    """Statement Object."""

    mainsnak: Snak
    qualifiers: Dict[str, List[Snak]]
    qualifiers_order: List[Property]
    type: Type
    rank: Rank
    references: List[Reference]


Statements = Dict[Property, List[StatementObject]]


class PricePoint(TypedDict):
    """Price in a specific currency + quantity.

    Based on https://octopart.com/api/v4/reference#pricepoint
    """

    # Minimum purchase quantity to get this price (aka price break).
    quantity: NotRequired[int]
    # Price in currency.
    price: NotRequired[float]
    # Currency for price.
    currency: NotRequired[str]
    # Currency for converted_price. Will match value of currency argument.
    converted_currency: NotRequired[str]
    # Price converted to user's currency using foreign exchange rates.
    converted_price: NotRequired[float]
    # The exchange rate used to calculate converted_price.
    conversion_rate: NotRequired[float]


class Offer(TypedDict):
    """A specific buyable part from a distributor.

    Based on https://octopart.com/api/v4/reference#offer
    """

    # Stock Keeping Unit used internally by distributor.
    sku: NotRequired[str]
    # The geo-political region(s) for which the offer is valid.
    eligible_region: NotRequired[Optional[str]]
    # Number of units available to be shipped (aka Stock, Quantity).
    inventory_level: NotRequired[int]
    # Packaging of parts (eg Tape, Reel).
    packaging: NotRequired[Optional[str]]
    # Minimum Order Quantity: smallest number of parts that can be
    # purchased.
    moq: NotRequired[int]
    prices: NotRequired[List[PricePoint]]
    # The URL to view offer on distributor website. This will
    # redirect via Octopart's server.
    click_url: NotRequired[str]
    # The last time data was fetched from external sources.
    updated_at: NotRequired[Optional[datetime]]
    # Number of days to acquire parts from factory.
    factory_lead_days: NotRequired[Optional[int]]
    # Number of parts on order from factory.
    on_order_quantity: NotRequired[Optional[int]]
    # Order multiple for factory orders.
    factory_pack_quantity: NotRequired[Optional[int]]
    # Number of items which must be ordered together.
    order_multiple: NotRequired[Optional[int]]
    # The quantity of parts as packaged by the seller.
    multipack_quantity: NotRequired[Optional[int]]


class Entity(TypedDict):
    """Entity."""

    aliases: NotRequired[Aliases]
    id: NotRequired[str]
    internal: NotRequired[Dict[str, Any]]
    labels: NotRequired[Labels]
    modified: NotRequired[datetime]
    statements: NotRequired[Statements]


class Seller(Entity):
    """Seller entity."""


class OfferGroup(TypedDict):
    """Offer group."""

    seller: Seller
    offers: List[Offer]


class Product(Entity):
    """Product entity."""

    offers: NotRequired[List[OfferGroup]]



"""Functions for constructing a KB entity."""
# Standard Modules
from typing import Dict, List, Optional

# Local Modules
from app.kb.entity.types import (
    Property,
    Rank,
    Reference,
    Snak,
    StatementObject,
    Type,
)


def statement_object(  # pylint: disable=too-many-arguments
    mainsnak: Snak,
    qualifiers: Optional[Dict[str, List[Snak]]] = None,
    qualifers_order: Optional[List[Property]] = None,
    type: Type = "statement",  # pylint: disable=redefined-builtin
    rank: Rank = "normal",
    references: Optional[List[Reference]] = None,
) -> StatementObject:
    """Create statement object."""

    return StatementObject(
        mainsnak=mainsnak,
        qualifiers=qualifiers or {},
        qualifiers_order=qualifers_order or [],
        type=type,
        rank=rank,
        references=references or [],
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cofactr',
 'cofactr.kb.entity',
 'cofactr.schema',
 'cofactr.schema.flagship',
 'cofactr.schema.logistics']

package_data = \
{'': ['*']}

install_requires = \
['urllib3>=1.26.9,<2.0.0']

setup_kwargs = {
    'name': 'cofactr',
    'version': '4.0.1',
    'description': 'Client library for accessing Cofactr data.',
    'long_description': '# Cofactr\n\nPython client library for accessing Cofactr.\n\n## Example\n\n```python\nfrom typing import List\nfrom cofactr.graph import GraphAPI\n\n# Flagship is the default schema.\nfrom cofactr.schema.flagship.part import Part\n\ngraph = GraphAPI()\n\npart_res = graph.get_product(id="IM60640MOX6H")\npart: Part = part_res["data"]\n\nparts_res = graph.get_products(query="esp32")\nparts: List[Part] = parts_res["data"]\n```\n',
    'author': 'Noah Trueblood',
    'author_email': 'noah@cofactr.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Cofactr/cofactr-client',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
